@@ -5,6 +5,7 @@
 # See PKCS11Session._healthy_session()
 
 Exposes the functions:
+- import_keypair()
 - create_keypair()
 - key_labels()
 - sign()
@@ -27,7 +28,7 @@ from asn1crypto.keys import (
 )
 from pkcs11.exceptions import NoSuchKey, SignatureInvalid, MultipleObjectsReturned
 from pkcs11 import KeyType, ObjectClass, Mechanism, lib, Session, Token, Attribute
-from pkcs11.util.rsa import encode_rsa_public_key
+from pkcs11.util.rsa import encode_rsa_public_key, decode_rsa_public_key, decode_rsa_private_key
 
 from .error import PKCS11TimeoutException, PKCS11UnknownErrorException
 
@@ -83,12 +84,71 @@ class PKCS11Session:
         if cls._session_status != 0:
             raise PKCS11UnknownErrorException("ERROR: Could not get a healthy PKCS11 connection")
 
+    @classmethod
+    def import_keypair(
+        cls, key_label: str, public_key: bytes, private_key: bytes
+    ) -> typing.Tuple[PublicKeyInfo, bytes]:
+        """Import a RSA keypair into the PKCS11 device with this label.
+        If the label already exists in the PKCS11 device then raise pkcs11.MultipleObjectsReturned.
+
+        Generating public_key and private_key can be done with:
+        openssl genrsa -out rsaprivkey.pem 2048
+        openssl rsa -inform pem -in rsaprivkey.pem -outform der -out PrivateKey.der
+        openssl rsa -in rsaprivkey.pem -RSAPublicKey_out -outform DER -out PublicKey.der
+
+        Returns the data for the x509 'Subject Public Key Info'
+        and x509 extension 'Subject Key Identifier' valid for this keypair.
+
+        Parameters:
+        key_label (str): Keypair label.
+        public_key (bytes): Public RSA key in DER form
+        private_key (bytes): Private RSA key in DER form
+
+        Returns:
+        typing.Tuple[asn1crypto.keys.PublicKeyInfo, bytes]
+        """
+
+        with LOCK:
+
+            # Ensure we get a healthy pkcs11 session
+            cls._healthy_session()
+
+            try:
+                key_pub = cls.session.get_key(
+                    key_type=KeyType.RSA,
+                    object_class=ObjectClass.PUBLIC_KEY,
+                    label=key_label,
+                )
+                raise MultipleObjectsReturned
+            except NoSuchKey as ex:
+                pass
+
+            key_pub = decode_rsa_public_key(public_key)
+            key_priv = decode_rsa_private_key(private_key)
+
+            key_pub[Attribute.TOKEN] = True
+            key_pub[Attribute.LABEL] = key_label
+            key_priv[Attribute.TOKEN] = True
+            key_priv[Attribute.LABEL] = key_label
+
+            cls.session.create_object(key_pub)
+            cls.session.create_object(key_priv)
+
+            # Create the PublicKeyInfo object
+            rsa_pub = RSAPublicKey.load(encode_rsa_public_key(key_pub))
+            pki = PublicKeyInfo()
+            pka = PublicKeyAlgorithm()
+            pka["algorithm"] = PublicKeyAlgorithmId("rsa")
+            pki["algorithm"] = pka
+            pki["public_key"] = rsa_pub
+            return pki, hashlib.sha1(encode_rsa_public_key(key_pub)).digest()
+
     @classmethod
     def create_keypair(
         cls, key_label: str, key_size: int = 2048
     ) -> typing.Tuple[PublicKeyInfo, bytes]:
         """Create a RSA keypair in the PKCS11 device with this label.
-        If the label exists then return the data for that keypair.
+        If the label already exists in the PKCS11 device then raise pkcs11.MultipleObjectsReturned.
         Returns the data for the x509 'Subject Public Key Info'
         and x509 extension 'Subject Key Identifier' valid for this keypair.
 
@@ -123,7 +183,6 @@ class PKCS11Session:
 
             # Create the PublicKeyInfo object
             rsa_pub = RSAPublicKey.load(encode_rsa_public_key(key_pub))
-
             pki = PublicKeyInfo()
             pka = PublicKeyAlgorithm()
             pka["algorithm"] = PublicKeyAlgorithmId("rsa")
